# -*- coding: utf-8 -*-
# _utils.py
# Miscellaneous functions and stdlib wrappers for MATLAB functions
# that do not find a direct replacement in numpy/scipy.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

""" Miscellaneous functions and wrappers for MATLAB functions
 that do not find a direct replacement in numpy/scipy.
"""

import collections
import fractions
from fractions import Fraction as Fr

import numpy as np
from scipy.signal import lti, ss2tf, ss2zpk, zpk2tf

from ._constants import eps
from ._partitionABCD import partitionABCD


def rat(x, tol):
    """Rational fraction approximation.

    Calculate A and B such that:

    .. math::

      x = \\frac{A}{B} + \\epsilon

    where:

    .. math::

        |\\epsilon| < tol


    .. note:: A, B are of type 'int'
    """
    return Fr(float(x)).limit_denominator(int(1 / float(tol))).numerator, \
        Fr(float(x)).limit_denominator(int(1 / float(tol))).denominator

gcd = fractions.gcd

lcm = lambda a, b: int(a * b / float(gcd(a, b)))
lcm.__doc__ = """Calculate the Least Common Multiple of ``a`` and ``b``.\n"""


class empty(object):

    """An empty function used to hold attributes"""

    def __init__(self):
        pass


def mfloor(x):
    """Round ``x`` towards -Inf.

    This is a MATLAB-compatible floor function, numpy's ``floor()``
    behaves differently.

    If the elements of ``x`` are complex, real and imaginary parts are
    rounded separately.
    """
    iform = save_input_form(x)
    x = carray(x)

    def _mfloor(z):
        """Base function to generate the ufunc floor"""
        z = np.real_if_close(z)
        if np.iscomplex(z):
            return _mfloor(np.real(z)) + 1j * _mfloor(np.imag(z))
        return np.floor(z) if np.sign(z) >= 0 else -np.ceil(-z)
    _internal = np.frompyfunc(_mfloor, 1, 1)
    xf = np.array(_internal(x), dtype=x.dtype)
    return restore_input_form(xf, iform)


def carray(x):
    """Check that x is an ndarray. If not, try to convert it to ndarray.
    """
    if not isinstance(x, np.ndarray):
        if not isinstance(x, collections.Iterable):
            x = np.array((x,))
        else:
            x = np.array(x)
    elif not len(x.shape):
        x = x.reshape((1,))
    else:
        pass  # nothing to do here
    return x


def cplxpair(x, tol=100):
    """
    Sort complex numbers into complex conjugate pairs.

    This function replaces MATLAB's cplxpair for vectors.
    """
    x = carray(x)
    x = np.atleast_1d(x.squeeze())
    x = x.tolist()
    x = [np.real_if_close(i, tol) for i in x]
    xreal = np.array(list(filter(np.isreal, x)))
    xcomplex = np.array(list(filter(np.iscomplex, x)))
    xreal = np.sort_complex(xreal)
    xcomplex = np.sort_complex(xcomplex)
    xcomplex_ipos = xcomplex[xcomplex.imag > 0.]
    xcomplex_ineg = xcomplex[xcomplex.imag <= 0.]
    if len(xcomplex_ipos) != len(xcomplex_ineg):
        raise ValueError("Complex numbers can't be paired.")
    res = []
    for i, j in zip(xcomplex_ipos, xcomplex_ineg):
        if not abs(i - np.conj(j)) < tol * eps:
            raise ValueError("Complex numbers can't be paired.")
        res += [j, i]
    return np.hstack((np.array(res), xreal))


def minreal(tf, tol=None):
    """Remove pole/zero pairs from a transfer function
    when the two match within the tolerance tol.

    tf may be a transfer function (lti object) or a list of transfer functions
    tol is optional and defaults to the system epsilon if unset.

    return a list of tfs or a tf, depending on the input type.
    """
    # initially based on python-control
    # which is in turn based on octave minreal
    # then modified considerably

    # recursively handle multiple tfs
    if not _is_zpk(tf) and not _is_num_den(tf) and not _is_A_B_C_D(tf) \
            and (isinstance(tf, list) or isinstance(tf, tuple)):
        ret = []
        for tfi in tf:
            ret += [minreal(tfi, tol)]
        return ret

    # default accuracy
    sqrt_eps = np.sqrt(eps)

    if (hasattr(tf, 'inputs') and not tf.inputs == 1) or \
       (hasattr(tf, 'outputs') and not tf.outputs == 1):
        raise TypeError("Only SISO transfer functions can be evaluated.")
    if hasattr(tf, 'zeros') and hasattr(tf, 'poles'):
        # LTI objects have poles and zeros,
        zeros = tf.zeros
        poles = tf.poles
        if hasattr(tf, 'k'):
            k = tf.k
        elif hasattr(tf, 'gain'):
            k = tf.gain
    else:
        # k = num[0] / den[0]
        zeros, poles, k = _get_zpk(tf)

    zeros = carray(zeros)
    poles = carray(poles)
    zeros.sort()
    poles.sort()
    reducedzeros = []

    for z in zeros:
        t = tol or 1000 * max(eps, abs(z) * sqrt_eps)
        idx = np.where(abs(poles - z) < t)[0]
        if len(idx):
            # cancel this zero against one of the poles
            # remove the pole and do not add the zero to the new
            poles = np.delete(poles, idx[0])
        else:
            # no matching pole
            reducedzeros.append(z)
    if len(reducedzeros):
        newzeros = carray(reducedzeros)
        num = k * np.real(np.poly(newzeros))
    else:
        num = np.array([k])
    den = np.real(np.poly(poles))

    return lti(num, den)


def diagonal_indices(a, offset=0):
    """Return the indices to the main diagonal of a 2D array a (if offset = 0),
    or to a secondary diagonal, having the offset from the main one as specified.

    The array a does not need to be square.

    Note: the sup-diagonal is at offset +1, the sub-diagonal at offset -1.

    Returns: (xs, ys)
    """
    di, dj = np.diag_indices_from(a[:min(a.shape), :min(a.shape)])
    if offset > 0:
        di, dj = zip(*[(i, j)
                     for i, j in zip(di, dj + offset) if 0 <= j < a.shape[1]])
    elif offset == 0:
        pass
    else:
        di, dj = zip(*[(i, j)
                     for i, j in zip(di - offset, dj) if 0 <= i < a.shape[0]])
    return di, dj


def circshift(a, shift):
    """Shift an array circularly.

    The ``circshift(a, shift)`` function circularly shifts the values in the
    array ``a`` by ``shift`` elements.

    **Parameters:**

    a : ndarray
        the array to be shifted. Notice that a should have a greater or equal
        number of dimensions than ``shift`` (``shift`` being a scalar is equal
        to ``shift`` being a one-dimension array.)

    shift : int or ndarray-like of int.
        the N-th element specifies the shift amount for the N-th dimension
        of the input array ``a``.

    If an element in ``shift`` is positive, the values of ``a`` are
    shifted to higher-index rows (ie down) or to higher-index columns
    (ie to the right).

    If the element is negative, the values of ``a`` are shifted in the opposite
    directions, towards lower-index rows (ie up) or to lower-index columns
    (ie right).

    If ``shift`` is an integer, the shift happens along axis 0.

    All dimensions that do not have a corresponding shift value in ``shift``
    are left untouched (ie ``shift=(1, 0, 0)`` is equal to ``shift=(1,)``,
    with the exception that the former will trigger an ``IndexError``
    if ``a.ndim < 3``).

    **Returns:**

    The shifted array.
    """
    if np.isscalar(shift):
        shift = [shift]
    for axis, ashift in enumerate(shift):
        a = np.roll(a, ashift, axis=axis)
    return a


def save_input_form(a):
    """Save the form of `a` so that it can be restored later on

    Returns: an object representing the form of `a`, to be passed to
                 restore_input_form(a, form)
    """
    if np.isscalar(a):
        ret = 'scalar'
    elif isinstance(a, np.ndarray):
        ret = a.shape
    elif type(a) == tuple:
        ret = 'tuple'
    elif type(a) == list:
        ret = 'list'
    else:
        raise TypeError("Unsupported input %s" % repr(a))
    return ret


def restore_input_form(a, form):
    """Restore the form of `a` according to `form`.

    Returns: the object `a`, in the correct `form`.

        Note: use `save_input_form(a)` to get the object `form`
    """
    if form == 'scalar':
        a = np.real_if_close(a)
        if not np.isscalar(a):
            a = a.reshape((1, ))[0]
    elif form == 'tuple':
        if not type(a) == tuple:
            a = [np.real_if_close(i).reshape((1,))[0] for i in a]
            a = tuple(a)
    elif form == 'list':
        if not type(a) == list:
            a = [np.real_if_close(i).reshape((1,))[0] for i in a]
            a = list(a)
    else:
        a = a.reshape(form)
    return a


def pretty_lti(arg):
    """Given the lti object ``arg`` return a *pretty* representation."""
    z, p, k = _get_zpk(arg)
    z = np.atleast_1d(z)
    p = np.atleast_1d(p)
    z = np.round(np.real_if_close(z), 4)
    p = np.round(np.real_if_close(p), 4)
    signs = {1:'+', -1:'-'}
    if not len(z) and not len(p):
        return "%g" % k
    ppstr = ["", "", ""]
    if k != 1:
        ppstr[1] = "%g " % k
    for i, s in zip((0, 2), (z, p)):
        rz = None
        m = 1
        sorted_singularities = cplxpair(s)
        for zindex, zi in enumerate(sorted_singularities):
            zi = np.round(np.real_if_close(zi), 4)
            if np.isreal(zi):
                if len(sorted_singularities) > zindex + 1 and \
                    sorted_singularities[zindex + 1] == zi:
                    m += 1
                    continue
                if zi == 0.:
                    ppstr[i] += "z"
                else:
                    ppstr[i] += "(z %s %g)" % (signs[np.sign(-zi)], np.abs(zi))
                if m == 1:
                    ppstr[i] += " "
                else:
                    ppstr[i] += "^%d " % m
                m = 1
            else:
                if len(sorted_singularities) > zindex + 2 and \
                    sorted_singularities[zindex + 2] == zi:
                    m += .5
                    continue
                if rz is None:
                    rz = zi
                    continue
                ppstr[i] += "(z^2 %s %gz %s %g)" % \
                            (signs[np.sign(np.real_if_close(np.round(-rz - zi, 3)))],
                             np.abs(np.real_if_close(np.round(-rz - zi, 3))),
                             signs[np.sign(np.real_if_close(np.round(rz * zi, 4)))],
                             np.abs(np.real_if_close(np.round(rz * zi, 4))))
                if m == 1:
                    ppstr[i] += " "
                else:
                    ppstr[i] += "^%d " % m
                rz = None
                m = 1
        ppstr[i] = ppstr[i][:-1] if len(ppstr[i]) else "1"
    if ppstr[2] == '1':
        return ppstr[1] + ppstr[0]
    else:
        if ppstr[0] == '1' and len(ppstr[1]) and float(ppstr[1]) != 1.:
            ppstr[0] = ppstr[1][:-1]
            ppstr[1] = ""
        space_pad_ln = len(ppstr[1])
        fraction_line = "-" * (max(len(ppstr[0]), len(ppstr[2])) + 2)
        ppstr[1] += fraction_line
        ppstr[0] = " "*space_pad_ln + ppstr[0].center(len(fraction_line))
        ppstr[2] = " "*space_pad_ln + ppstr[2].center(len(fraction_line))
    return "\n".join(ppstr)


def _get_zpk(arg, input=0):
    """Utility method to convert the input arg to a z, p, k representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    input : scalar
        In case the system has multiple inputs, which input is to be
        considered. Input `0` means first input, and so on.

    **Returns:**

    The sequence of ndarrays z, p and a scalar k

    **Raises:**

    TypeError, ValueError

    .. warn: support for MISO transfer functions is experimental.
    """
    z, p, k = None, None, None
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
        z, p, k = ss2zpk(A, B, C, D, input=input)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
        z, p, k = arg.zeros, arg.poles, arg.gain
    elif _is_zpk(arg):
        z, p, k = np.atleast_1d(arg[0]), np.atleast_1d(arg[1]), arg[2]
    elif _is_num_den(arg):
        sys = lti(*arg)
        z, p, k = sys.zeros, sys.poles, sys.gain
    elif _is_A_B_C_D(arg):
        z, p, k = ss2zpk(*arg, input=input)
    elif isinstance(arg, collections.Iterable):
        ri = 0
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched lti representations.
            if hasattr(i, 'B'):
                iis = i.B.shape[1]
                if input < ri + iis:
                    z, p, k = ss2zpk(i.A, i.B, i.C, i.D, input=input - ri)
                    break
                else:
                    ri += iis
                    continue
            elif _is_A_B_C_D(arg):
                iis = arg[1].shape[1]
                if input < ri + iis:
                    z, p, k = ss2zpk(*arg, input=input - ri)
                    break
                else:
                    ri += iis
                    continue
            else:
                if ri == input:
                    sys = lti(*i)
                    z, p, k = sys.zeros, sys.poles, sys.gain
                    break
                else:
                    ri += 1
                    continue
                ri += 1
        if (z, p, k) == (None, None, None):
            raise ValueError("The LTI representation does not have enough" +
                             "inputs: max %d, looking for input %d" %
                             (ri - 1, input))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)
    return z, p, k


def _get_num_den(arg, input=0):
    """Utility method to convert the input arg to a (num, den) representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    input : scalar
        In case the system has multiple inputs, which input is to be
        considered. Input `0` means first input, and so on.

    **Returns:**

    The sequence of ndarrays num, den

    **Raises:**

    TypeError, ValueError

    .. warn: support for MISO transfer functions is experimental.
    """
    num, den = None, None
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
        num, den = ss2tf(A, B, C, D, input=input)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
        num, den = arg.num, arg.den
    elif _is_num_den(arg):
        num, den = carray(arg[0]).squeeze(), carray(arg[1]).squeeze()
    elif _is_zpk(arg):
        num, den = zpk2tf(*arg)
    elif _is_A_B_C_D(arg):
        num, den = ss2tf(*arg, input=input)
    elif isinstance(arg, collections.Iterable):
        ri = 0
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched representations.
            if hasattr(i, 'B'):  # lti
                iis = i.B.shape[1]
                if input < ri + iis:
                    num, den = ss2tf(i.A, i.B, i.C, i.D, input=input - ri)
                    break
                else:
                    ri += iis
            else:
                sys = lti(*i)
                iis = sys.B.shape[1]
                if input < ri + iis:
                    num, den = ss2tf(
                        sys.A, sys.B, sys.C, sys.D, input=input - ri)
                    break
                else:
                    ri += iis

        if (num, den) == (None, None):
            raise ValueError("The LTI representation does not have enough" +
                             "inputs: max %d, looking for input %d" %
                             (ri - 1, input))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)

    if len(num.shape) > 1:
        num = num.squeeze()
    if len(den.shape) > 1:
        den = den.squeeze()

    # default accuracy: sqrt_ps
    sqrt_eps = np.sqrt(eps)
    while len(num.shape) and len(num):
        if abs(num[0]) < sqrt_eps:
            num = num[1:]
        else:
            break
    while len(den.shape) and len(den):
        if abs(den[0]) < sqrt_eps:
            den = den[1:]
        else:
            break
    den = np.atleast_1d(den)
    num = np.atleast_1d(num)

    return num, den


def _getABCD(arg):
    """Utility method to convert the input arg to an A, B, C, D representation.

    **Parameters:**

    arg, which may be:

    * ZPK tuple,
    * num, den tuple,
    * A, B, C, D tuple,
    * a scipy LTI object,
    * a sequence of the tuples of any of the above types.

    **Returns:**

    The sequence of ndarrays A, B, C, D

    **Raises:**

    TypeError, ValueError
    """
    if isinstance(arg, np.ndarray):
        # ABCD matrix
        A, B, C, D = partitionABCD(arg)
    elif hasattr(arg, '__class__') and arg.__class__.__name__ == 'lti':
        A, B, C, D = arg.A, arg.B, arg.C, np.atleast_2d(arg.D)
    elif _is_zpk(arg) or _is_num_den(arg) or _is_A_B_C_D(arg):
        sys = lti(*arg)
        A, B, C, D = sys.A, sys.B, sys.C, sys.D
    elif isinstance(arg, collections.Iterable):
        A, B, C, D = None, None, None, None
        for i in arg:
            # Note we do not check if the user has assembled a list with
            # mismatched lti representations.
            sys = lti(*i) if not hasattr(i, 'A') else i
            if A is None:
                A = sys.A
            elif not np.allclose(sys.A, A, atol=1e-8, rtol=1e-5):
                raise ValueError("Mismatched lti list, A matrix disagreement.")
            else:
                pass
            if B is None:
                B = sys.B
            else:
                B = np.hstack((B, sys.B))
            if C is None:
                C = sys.C
            elif not np.allclose(sys.C, C, atol=1e-8, rtol=1e-5):
                raise ValueError("Mismatched lti list, C matrix disagreement.")
            if D is None:
                D = sys.D
            else:
                D = np.hstack((D, sys.D))
    else:
        raise TypeError("Unknown LTI representation: %s" % arg)
    return A, B, C, D


def _is_zpk(arg):
    """Can the argument be safely assumed to be a zpk tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 3 and \
        isinstance(arg[0], collections.Iterable) and \
        isinstance(arg[1], collections.Iterable) and np.isscalar(arg[2])


def _is_num_den(arg):
    """Can the argument be safely assumed to be a num, den tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 2 and \
        isinstance(arg[0], collections.Iterable) and \
        isinstance(arg[1], collections.Iterable)


def _is_A_B_C_D(arg):
    """Can the argument be safely assumed to be an (A, B, C, D) tuple?"""
    return isinstance(arg, collections.Iterable) and len(arg) == 4 and \
        (isinstance(arg[0], collections.Iterable) or np.is_scalar(arg[0])) and \
        (isinstance(arg[1], collections.Iterable) or np.is_scalar(arg[1])) and \
        (isinstance(arg[2], collections.Iterable) or np.is_scalar(arg[2])) and \
        (isinstance(arg[3], collections.Iterable) or np.is_scalar(arg[3]))


def _cell_like_list(shape, init=None):
    """Returns a list of lists (possibly of lists... etc...),
    with all values initialized to `init`.

    **Parameters:**

    shape: tuple of ints,
           the dimensions of the desidered object

    init: object, optional,
          the initialization value for every element in the object

    **Returns:**

    cell, a list of lists (...)

    .. deprecated:: 0.1-3

    """
    a = []
    for _ in range(shape[0]):
        if len(shape) == 1:
            a.append(init)
        else:
            a.append(_cell_like_list(shape[1:], init))
    return a


def mround(x):
    """Round ``x`` to the nearest integers.

    This is a MATLAB-compatible round function, numpy's ``round()``
    behaves differently.

    Behaviour with a fractional part of 0.5:

    * Positive elements with a fractional part of 0.5 round up to the nearest positive integer.

    * Negative elements with a fractional part of -0.5 round down to the nearest negative integer.

    If the elements of ``x`` are complex, real and imaginary parts are
    rounded separately.

    **Example:**

       >>> mround([-1.9, -0.5, -0.2, 3.4, 4.5, 5.6, 7.0, 2.4+3.6j])
       [-2.0, -1.0, 0.0, 3.0, 5.0, 6.0, 7.0, 2.0+4.0j]

    """
    iform = save_input_form(x)
    x = carray(x)

    def _mround(z):
        """Base function to generate the ufunc round"""
        z = np.real_if_close(z)
        if np.iscomplex(z):
            return _mround(np.real(z)) + 1j*_mround(np.imag(z))
        s = 1 if z >= 0 else -1
        res = z - s*(abs(z) % 1) if abs(z) % 1 < .5 \
              else z + s*(1 - (abs(z)%1))
        return res
    _internal = np.frompyfunc(_mround, 1, 1)
    xf = np.array(_internal(x), dtype=x.dtype)
    return restore_input_form(xf, iform)



# -*- coding: utf-8 -*-
# test_infnorm.py
# This module provides the tests for the infnorm function.
# Copyright 2014 Giuseppe Venturini & Shayne Hodge
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

import unittest
import numpy as np
import deltasigma as ds


class TestInfNorm(unittest.TestCase):
    """Test functions for figureMagic()"""
    def setUp(self):
        num, den = np.poly([3, 0.3, 1]), np.poly([2, 0.5, .25])
        H = (num, den)
        self.Hinf, self.fmax = ds.infnorm(H)

    def test_infnorm_Hinf(self):
        """Test function for infnorm() checking Hinf"""
        self.assertTrue(np.allclose(
            self.Hinf, 1.84888889, atol=1e-8, rtol=1e-5))

    def test_infnorm_fmax(self):
        """Test function for infnorm() checking fmax"""
        self.assertTrue(np.allclose(
            self.fmax, 3.141592653589793/2.0/np.pi, atol=1e-8, rtol=1e-5))



# -*- coding: utf-8 -*-
# __init__.py
# python-deltasigma module init file
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""

:Author: Giuseppe Venturini
:Release: |release|
:Date: |today|

**Homepage:** http://www.python-deltasigma.io

**Documentation:** http://docs.python-deltasigma.io

**Repository:** https://github.com/ggventurini/python-deltasigma

**Bug tracker:** https://github.com/ggventurini/python-deltasigma/issues

Introduction
------------

A port of the **MATLAB Delta Sigma Toolbox** based on free software and
very little sleep.

**Python-deltasigma** is a Python package to *synthesize, simulate,
scale and map to implementable structures* **delta sigma modulators**.

It aims to provide **a 1:1 Python port** of Richard Schreier's
***excellent*** `MATLAB Delta Sigma
Toolbox <http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox>`__,
the *de facto* standard tool for high-level delta sigma simulation, upon
which it is very heavily based.\ |githalytics.com alpha|

Status
------

|Build Status| |Coverage Status| |PyPi version| |PyPi downloads| |BSD
2 clause license| |DOI BADGE|

This project is a *work in progress*, not all functionality has been
ported, yet. The next figure shows the relationship between the main functions
and the avaliable functionality at a glance.

.. image:: ../doc/_static/functionality.png

All the basic features are available since v. 0.1. A detailed changelog may be
found in `CHANGES.rst <https://github.com/ggventurini/python-deltasigma/blob/master/CHANGES.rst>`__.

Detailed information split by file and function status may be found in
`files.csv <https://github.com/ggventurini/python-deltasigma/blob/master/files.csv>`__.

The further functionality is expected to be ported and available in future releases
according to `the ROADMAP <https://github.com/ggventurini/python-deltasigma/blob/master/ROADMAP.md>`__.

Examples
--------

To see the currently implemented functionality in action, take a look
at the following ipython notebooks:

-  `dsdemo1 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsdemo1.ipynb>`__,
   notebook port of the interactive ``dsdemo1.m``.
-  `dsdemo2 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsdemo2.ipynb>`__,
   notebook port of the interactive ``dsdemo2.m``.
-  `dsdemo3 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsdemo3.ipynb>`__,
   notebook port of the interactive ``dsdemo3.m``.
-  `dsdemo4 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsdemo4.ipynb>`__,
   notebook port of ``dsdemo4.m``. `Audio
   file <https://raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/sax.wav.b64>`__, right click to download.
-  `dsexample1 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsexample1.ipynb>`__, python
   version of ``dsexample1.m``.
-  `dsexample2 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsexample2.ipynb>`__, python
   version of ``dsexample2.m``.
-  `dsexample3 <http://nbviewer.ipython.org/urls/raw.githubusercontent.com/ggventurini/python-deltasigma/master/examples/dsexample3.ipynb>`__, python
   version of ``dsexample3.m``.

They are also a good means for getting started quickly.

If you have some examples you would like to share, `send me a
mail <http://tinymailto.com/5310>`__, and I will add them to the above
list.

Install
-------

Supported platforms
~~~~~~~~~~~~~~~~~~~

python-deltasigma runs on every platform and arch. supported by its
dependencies:

-  *Platforms*: Linux, Mac OS X, Windows.

-  *Archs*: x86, x86\_64 and armf (arm with floating point unit).

Dependencies
~~~~~~~~~~~~

Using ``python-deltasigma`` requires `Python 2 or 3 <http://www.python.org/>`__,
at your choice, `numpy <http://www.numpy.org/>`__,
`scipy <http://www.scipy.org>`__ (>= 0.11.0) and
`matplotlib <http://www.matplotlib.org>`__.

They are packaged by virtually all the major Linux distributions.

On a Debian Linux system, you may install them issuing:

::

    aptitude install python python-numpy python-scipy python-matplotlib

Refer to your system documentation for more information.

On Windows, I hear good things about:

-  `Enthought Canopy <https://www.enthought.com/store/>`__, a Python
   distribution that carries both free and commercial versions, and

-  `Anaconda <https://store.continuum.io/cshop/anaconda/>`__, which
   offers its full version for free.

I do not run Windows, so I can't really provide more info (sorry),
except that people tell me they manage to have a working setup.

*Mac OS X* is also supported by `Enthought
Canopy <https://www.enthought.com/store/>`__ and
`Anaconda <https://store.continuum.io/cshop/anaconda/>`__, which likely
provide the easiest and fastest solution to have a scientific Python
stack up and running.

More information can be found on the `scipy install
page <http://www.scipy.org/install.html>`__ and on the `matplotlib
homepage <http://matplotlib.org/>`__.

I wrote in a different context some directions to `compile numpy and
scipy
yourself <https://github.com/ahkab/ahkab/wiki/Install:-numpy-and-scipy>`__,
which also apply here. Be warned, it can easily get complicated.

Optional dependencies
~~~~~~~~~~~~~~~~~~~~~

The required dependencies have been kept to a minimum to allow running
``python-deltasigma`` on workstations that are not managed by the user
but by a system administrator - where typically installing libraries is
not possible and software packages are disarmingly outdated.

If at all possible, installing `Cython <http://www.cython.org>`__ is
strongly recommended.

``python-deltasigma`` contains python extension to simulate delta sigma
modulators providing a near-native execution speed -- overall roughly a
70x speed-up compared to a plain Python implementation.

On Linux, installing Cython is just one: `aptitude install cython`
away.

On Mac OS X and Windows, Cython may be installed as part of one of the
frameworks above. Please notice a compiler is needed, this may require
installing XCode and its command-line utilities or gcc through homebrew,
on Mac OS X, or Mingw, on Windows.

If the BLAS headers are found on the machine, they will be used. In
case they cannot be found automatically, it is recommended to set
the environment variable ``BLAS_H`` to the BLAS headers directory.

On Mac OS X, consider linking the headers to their conventional location::

    sudo ln -s /System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/Current/Headers/cblas.h /usr/include/cblas.h

The Cython extensions were written by Sergio Callegari, please see the
``deltasigma/`` for copyright notice and more information.

Install python-deltasigma
~~~~~~~~~~~~~~~~~~~~~~~~~

Once the dependencies set up, it is possible to install the latest stable
version directly from the `Python Package Index (PYPI)
<http://pypi.python.org>`__, running::

    pip install deltasigma

The above command will also attempt to compile and install the dependencies
in case they are not found. Please notice this is not recommended and
for this to work you should already have the required C libraries in place.

If you are interested in a bleeding-edge version -- potentially less stable
-- or in contributing code (*that's awesome!*) you can head over to
`the Github repository <http://github.com/ggventurini/python-deltasigma>`__
and check out the code from there.

Then run::

    python setup.py install

The flag ``--user`` may be an interesting option to install the package for
the current user only, and it doesn't require root privileges.

Extras for developers
~~~~~~~~~~~~~~~~~~~~~

The following may be installed at a later stage and are typically only
necessary for developers.

Building the documentation requires the
`sphinx <http://sphinx-doc.org/>`__ package. It is an optional step,
as the `the latest documentation is available
online <http://python-deltasigma.readthedocs.org/en/latest/>`__, without
need for you to build it.

If you plan to modify the code, `python-deltasigma` comes with a complete
unit tests suite, which is run against every commit and that any addition
should pass both for Python 2 and 3.
To run it locally, `setuptools <https://pypi.python.org/pypi/setuptools>`__
is needed, as it is used to access the reference function outputs.

Running the test suite may be conveniently automated installing
`nose <https://pypi.python.org/pypi/nose/>`__, and then issuing::

    nosetests -v deltasigma

from the repository root.

Useful resources
----------------

The original MATLAB Toolbox provides in-depth documentation, which is
very useful to understand what the toolbox is capable of. See
`DSToolbox.pdf <https://github.com/ggventurini/python-deltasigma/blob/master/delsig/DSToolbox.pdf?raw=true>`__
and
`OnePageStory.pdf <https://github.com/ggventurini/python-deltasigma/blob/master/delsig/OnePageStory.pdf?raw=true>`__
(*PDF warning*).

The book:

    Richard Schreier, Gabor C. Temes, *Understanding Delta-Sigma Data
    Converters*, ISBN: 978-0-471-46585-0, November 2004, Wiley-IEEE Press

is probably *the most authoritative resource on the topic*. Chapter 8-9
show how to use the MATLAB toolkit and the observations apply also to
this Python port. Links on
`amazon <http://www.amazon.com/Understanding-Delta-Sigma-Converters-Richard-Schreier/dp/0471465852>`__,
on `the Wiley-IEEE
press <http://eu.wiley.com/WileyCDA/WileyTitle/productCd-0471465852,miniSiteCd-IEEE2.html>`__.

*I am not affiliated with neither the sellers nor the authors.*

How to contribute
-----------------

Pull requests are welcome!
~~~~~~~~~~~~~~~~~~~~~~~~~~

If you want to port some code, fix a bug, add a cool example or implement new
functionality (in this case it may be a good idea to get in touch early on),
*that's awesome!*

There are only a few *guidelines*, which can be overridden every time it
is reasonable to do so:

-  Please try to follow ``PEP8``.

-  Try to keep the functions signature identical. Parameters with
   ``NaN`` default values have their default value replaced with
   ``None``.

-  If a function has a varible number of return values, its Python port
   should implement the maximum number of return values.

No commit should ever fail the test suite.

Reporting bugs
~~~~~~~~~~~~~~

What bugs, *there are no bugs!*

Jokes aside, please report all bugs on `on the Github issue tracker <https://github.com/ggventurini/python-deltasigma/issues>`__.

Support python-deltasigma
~~~~~~~~~~~~~~~~~~~~~~~~~

*I do not want your money.* I develop this software because I enjoy it
and because I use it myself.

If you wish to support the development of ``python-deltasigma`` or you
find the package useful or you otherwise wish to contribute monetarily,
***please donate to cancer research instead:***

-  `Association for International Cancer Research
   (eng) <http://www.aicr.org.uk/donate.aspx>`__, or
-  `Fond. IRCCS Istituto Nazionale dei Tumori
   (it) <http://www.istitutotumori.mi.it/modules.php?name=Content&pa=showpage&pid=24>`__.

Consider `sending me a mail <http://tinymailto.com/5310>`__ afterwards,
***it makes for great motivation!***

License, copyright, rationale and credits
-----------------------------------------

Why this project was born
~~~~~~~~~~~~~~~~~~~~~~~~~

I like challenges, delta-sigma modulation and I don't have the money for
my own MATLAB license. After all, *which grad student or young researcher
has it?*

With this Python package you can simulate delta-sigma modulators for
free, on any PC.

I hope you find it useful.

Licensing and copyright notice
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

All original MATLAB code is Copyright (c) 2009, Richard Schreier. See
the LICENSE file for the licensing terms.

The Python code here provided is a derivative work from the above
toolkit and subject to the same license terms.

Credit goes to Richard Schreier for the original ideas, their MATLAB
implementation and the all the  diagrams found in this documentation.
Little-to-no conceptual improvements are introduced here, just code
adaptation, refactoring, rewrites and fixing of a few minor issues.

This package contains some source code from ``pydsm``, also based on the
same MATLAB toolbox. The ``pydsm`` package is copyright (c) 2012, Sergio
Callegari.

When not otherwise specified, the Python code is Copyright 2013,
Giuseppe Venturini and the python-deltasigma contributors.

MATLAB is a registered trademark of The MathWorks, Inc.

.. |githalytics.com alpha| image:: https://cruel-carlota.pagodabox.com/36f25accf60f391456efe66910bf84f8
   :target: http://githalytics.com/ggventurini/python-deltasigma
   :width: 1
.. |Build Status| image:: https://travis-ci.org/ggventurini/python-deltasigma.png?branch=master
   :target: https://travis-ci.org/ggventurini/python-deltasigma
.. |Coverage Status| image:: https://coveralls.io/repos/ggventurini/python-deltasigma/badge.png?branch=master
   :target: https://coveralls.io/r/ggventurini/python-deltasigma?branch=master
.. |PyPi version| image:: http://img.shields.io/badge/version-0.1-brightgreen.png
   :target: https://pypi.python.org/pypi/deltasigma/
.. |PyPi downloads| image::  https://pypip.in/download/deltasigma/badge.png
   :target: https://pypi.python.org/pypi/deltasigma/
.. |BSD 2 clause license| image:: http://img.shields.io/badge/license-BSD-brightgreen.png
   :target: https://raw.githubusercontent.com/ggventurini/python-deltasigma/master/LICENSE
.. |DOI BADGE| image:: https://zenodo.org/badge/doi/10.5281/zenodo.11167.png
   :target: http://dx.doi.org/10.5281/zenodo.11167


Credits
~~~~~~~

The ``python-deltasigma`` package was written by
`Giuseppe Venturini <https://github.com/ggventurini>`__, as a derivative work
of Richard Schreier's MATLAB Delta Sigma toolbox. It contains code from
``pydsm``, also based on the same MATLAB toolbox and written by Sergio
Callegari.

Contributors: Shayne Hodge

Implementation model
--------------------

The internal implementation of delta sigma modulators follows closely the one in
Richard Schreier's `MATLAB Delta Sigma
Toolbox <http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox>`__,
upon which the following documentation is very heavily based.


Modulator model
~~~~~~~~~~~~~~~

A delta-sigma modulator with a single quantizer is assumed to consist of
quantizer connected to a loop filter as shown in the diagram below.

.. image:: ../doc/_static/modulator_model.png
    :align: center
    :alt: Modulator model

.. _loop-filter-label:

The loop filter
:::::::::::::::

The loop filter is described by an :math:`ABCD` matrix. For single-quantizer
systems, the loop filter is a two-input, one-output linear system and
:math:`ABCD` is an :math:`(n+1, n+2)` matrix, partitioned into
:math:`A` :math:`(n, n)`, :math:`B` :math:`(n, 2)`, :math:`C` :math:`(1, n)`
and :math:`D` :math:`(1, 2)` sub-matrices as shown below:

.. math::

    ABCD =
        \\left[
        \\begin{array}{c|c}
          A & B \\\\ \\hline
          C & D
        \\end{array}
        \\right].

The equations for updating the state and computing the output of the loop filter are:

.. math::

    x(n + 1) = Ax(n) + B
        \\left[
        \\begin{array}{c}
          u(n) \\\\
          v(n)
        \\end{array}
        \\right]

.. math::

    y(n) = Cx(n) + D
        \\left[
        \\begin{array}{c}
          u(n) \\\\
          v(n)
        \\end{array}
        \\right].

Where :math:`u(n)` is the input sequence and :math:`v(n)` is the modulator output sequence.

This formulation is sufficiently general to encompass all single-quantizer modulators which
employ linear loop filters. The toolbox currently supports translation to/from an ABCD descrip-
tion and coefficients for the following topologies:

 * CIFB : Cascade-of-integrators, feedback form.
 * CIFF : Cascade-of-integrators, feedforward form.
 * CRFB : Cascade-of-resonators, feedback form.
 * CRFF : Cascade-of-resonators, feedforward form.
 * CRFBD : Cascade-of-resonators, feedback form, delaying quantizer.
 * CRFFD : Cascade-of-resonators, feedforward form, delaying quantizer
 * PFF : Parallel feed-forward
 * Stratos : A CIFF-like structure with non-delaying resonator feedbacks [*]_

.. [*] Contributed to the MATLAB delta sigma toolbox in 2007 by Jeff Gealow.

See :ref:`topologies-diagrams` for a block-level view of the different modulator structures.

Multi-input and multi-quantizer systems can also be described with an
ABCD matrix and the previous equation
will still apply. For an :math:`n_i`-input, :math:`n_o`-output modulator,
the dimensions of the sub-matrices are
:math:`A`: :math:`(n, n)`, :math:`B`: :math:`(n, n_i + n_o)`,
:math:`C`: :math:`(n_o, n)` and :math:`D`: :math:`(n_o, n_i+n_o)`.

Quantizer model
:::::::::::::::

The quantizer is ideal, producing integer outputs centered about zero. Quantizers with an even
number of levels are of the mid-rise type and produce outputs which are odd integers. Quantizers
with an odd number of levels are of the mid-tread type and produce outputs which are even inte-
gers.

.. image:: ../doc/_static/quantizer_model.png
    :align: center
    :alt: Quantizer model

.. seealso:: :func:`bquantize`, :func:`bunquantize`

.. _topologies-diagrams:

Topologies diagrams
~~~~~~~~~~~~~~~~~~~

All the following topology diagrams are reproduced from
`DSToolbox.pdf <https://github.com/ggventurini/python-deltasigma/blob/master/delsig/DSToolbox.pdf?raw=true>`__
in the `MATLAB Delta Sigma Toolbox <http://www.mathworks.com/matlabcentral/fileexchange/19-delta-sigma-toolbox>`__,
written by Richard Schreier. All credits belong to the original author.

CIFB
::::

.. image:: ../doc/_static/CIFB.png
    :align: center
    :alt: CIFB topology

CIFF
::::

.. image:: ../doc/_static/CIFF.png
    :align: center
    :alt: CIFF topology

CRFB
::::

.. image:: ../doc/_static/CRFB.png
    :align: center
    :alt: CRFB topology

CRFF
::::

.. image:: ../doc/_static/CRFF.png
    :align: center
    :alt: CRFF topology

CRFBD
:::::

.. image:: ../doc/_static/CRFBD.png
    :align: center
    :alt: CRFBD topology

CRFFD
:::::

.. image:: ../doc/_static/CRFFD.png
    :align: center
    :alt: CRFFD topology

.. _discrete-time-to-continuous-time-mapping:

Discrete time to continuous time mapping
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The approach presented here to design a CT delta-sigma modulator starts with the
synthesis of the noise transfer function.

Once a suitable NTF has been identified, we need to realize the transfer function with a
continuous time loop filter.

First, a loop filter topology is selected among the Feed-Forward (FF) and Feedback (FB)
structures. The feed-forward or feedback paths -- depending on the topology -- will be
characterized by an unknown proportionality factor :math:`k_i`, for each of the
:math:`i \in \\{0 \dots order\\}` branches.

.. image:: ../doc/_static/DS_equivalence_DT_CT.png
    :align: center
    :alt: DT-CT mapping model

The objective is to determine the gain factors :math:`{k_i}` to construct a CT loop
filter such that its sampled pulse response is equal to the impulse response of a DT
prototype's loop filter.

We consider here three approaches:

* equating the loop filter transfer functions,
* matching the loop filter impulse responses, implemented in :func:`realizeNTF_ct`
  as method ``'LOOP'``,
* matching the DT NTF and CT filter pulse responses, implemented in
  :func:`realizeNTF_ct` as method ``'NTF'``.

1: equate the loop filter transfer functions
::::::::::::::::::::::::::::::::::::::::::::

The DT loop filter transfer function can be found from:

.. math::

    L_{1,DT}(z) = 1 - \\frac{1}{NTF_{DT}(z)}

The CT loop filter is readily known, since it has been selected by the user,
but it needs to be converted to an equivalent DT transformation. This operation
is performed through the impulse invariance transformation, here denoted as
:math:`\\mathscr{I\\!I\\!T}` [R2]_.

Solving the equation:

.. math::

    L_{1, DT}(z) = \\mathscr{I\\!I\\!T}\\!\\!\\left(L_{1, CT}\\right)(z)

will allow determining the exact values of the coefficients :math:`k_i` [R1]_.

This approach has limited use in the real case:

* Developing an analytical CT model and applying the impulse invariance
  transformation may entail significant difficulties in presence of
  non-idealities.
* The equation above will not have a solution when the integrators are non-ideal
  since the poles of the transfer functions on RHS and LHS are different.

2: match the loop filter impulse responses
::::::::::::::::::::::::::::::::::::::::::

Although it may be non-trivial to reach an analytical expression for
:math:`\\mathscr{I\\!I\\!T}\\!\\!\\left(L_{1, CT}\\right)(z)`, it is still
possible to evaluate through numerical simulations (or in a some case
analytically), N samples of the pulse response of each path
:math:`\\{l_i[n]\\}`, obtained setting all the gain coefficients to one
or :math:`k_i = 1, \\; \\forall i \\in \\{0 \\dots order\\}`.

Evaluating the impulse response of the prototype DT loop filter, denoted in the
following as :math:`l[n]`, it is possible to write the equation:

.. math::

    [\\ l_0[n]\\; l_1[n]\\; \\dots \\; l_{order}[n]\\ ]\ K = l[n]

Where we define the vector :math:`K` as:

.. math::

    K = [\\ k_0 \\; k_1 \\; \dots \\; k_{order}\\ ]^T

In the ideal case, provided that the impulse responses have been evaluated for
a sufficiently high number of points :math:`N` (:math:`N > order`), the
equation has a exact solution, indepently of :math:`N` [R2]_.

In presence of non-idealities, it is possible to use Least Squares fitting to
find the optimum :math:`\{k_i\}`.

As discussed in [R3]_, this method is particularly sensitive to the value of
:math:`N` in the non-ideal cases.

3: match the DT NTF and CT filter pulse responses
:::::::::::::::::::::::::::::::::::::::::::::::::

A more robust approach is the following:

Remember the goal of DT to CT mapping is to have:

.. math::

    NTF_{DT} = NTF_{eq, CT}

And by definition:

.. math::

    NTF = \\frac{1}{1+L_1(z)}

Combining the two we can write the equation:

.. math::

    NTF_{DT} = \\frac{1}{1+L_{1,eq,CT}(z)}

To avoid having to apply the impulse invariance transformation, we can
rewrite the above in the time domain, getting:

.. math::

    \\sum_i k_i \\left(h[n] * l_i[n]\\right) = \\delta[n] - h[n] \\qquad \\forall i

Where :math:`h[n]` is the impulse response of the DT NTF.

The above can be rewritten as:

.. math::

    [\\ h[n]*l_0[n] \\; \\dots \\; h[n]*l_{order}[n]\\ ] K = \\delta[n] - h[n]

And solved exactly, in the ideal case, or in the least squares sense,
in presence of non-idealities [R3]_.

.. [R1] P. Benabes, M. Keramat, and R. Kielbasa, "A methodology for designing
        continuous-time sigma-delta modulators," in Proc. Eur. Conf. Design
        Test, Washington, DC, 1997, pp. 46-50

.. [R2] J. Cherry and W. Snelgrove, "Excess loop delay in continuous-time
        delta-sigma modulators," IEEE Trans. Circuits Syst. II,
        Analog Digit. Signal Process., vol. 46, no. 4, pp. 376-389, Apr. 1999

.. [R3] Shanthi Pavan, "Systematic Design Centering of Continuous Time
        Oversampling Converters",  IEEE Trans. on Circuits Syst. II Express
        Briefs, Volume.57, Issue 3, pp.158, 2010

Package contents
----------------

Key Functions
~~~~~~~~~~~~~

.. autosummary::
    :nosignatures:

    synthesizeNTF
    clans
    synthesizeChebyshevNTF
    simulateDSM
    simulateSNR
    realizeNTF
    stuffABCD
    mapABCD
    scaleABCD
    calculateTF
    realizeNTF_ct
    mapCtoD
    evalTFP

Functions for quadrature systems
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Notice the current version of ``python-deltasigma`` cannot synthesize
quadrature modulators. This feature is expected in v. 0.2.

Nontheless, the following functions are provided since if you already have
a synthesized modulator (or know its ABCD matrix), they allow you to simulate
the modulator with the standard tools for real ABCDs topologies.

.. autosummary::
    :nosignatures:

    mapQtoR
    mapRtoQ

Other selected functions
~~~~~~~~~~~~~~~~~~~~~~~~

The following are auxiliary functions that complement the key functions above.

.. autosummary::
    :nosignatures:

    mod1
    mod2
    calculateSNR
    predictSNR
    partitionABCD
    infnorm
    impL1
    l1norm
    pulse
    rmsGain

Utility functions for simulation of delta-sigma modulators
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Functions for low-level handling of delta-sigma modulator representations,
their evaluation and filtering.

.. autosummary::
    :nosignatures:

    bquantize
    bunquantize
    cancelPZ
    circshift
    delay
    ds_f1f2
    ds_freq
    ds_hann
    ds_optzeros
    ds_quantize
    ds_synNTFobj1
    dsclansNTF
    evalMixedTF
    evalRPoly
    evalTF
    nabsH
    peakSNR
    sinc_decimate
    zinc

General utilities for data processing
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The following are generic functions, useful for misc. tasks, like
manipulating data, conversions or padding, for example.
They provide speciality functions which are not otherwise available
in the usual scientific Python stack.

.. autosummary::
    :nosignatures:

    db
    dbm
    dbp
    dbv
    undbm
    undbp
    undbv
    rms
    padb
    padl
    padr
    padt
    cplxpair
    mfloor
    mround
    rat
    gcd
    lcm

Plotting and data display utilitites
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Graphic functions:

.. autosummary::
    :nosignatures:

    plotPZ
    plotSpectrum
    figureMagic
    DocumentNTF
    PlotExampleSpectrum
    axisLabels
    bilogplot
    lollipop
    changeFig

Textual and non-graphic display-related functions

.. autosummary::
    :nosignatures:

    circ_smooth
    bplogsmooth
    logsmooth
    pretty_lti
    SIunits

All functions in alphabetical order
-----------------------------------

"""

__author__ = "Giuseppe Venturini and the python-deltasigma contributors"
__copyright__ = "Copyright 2013, Giuseppe Venturini"
__credits__ = ["Giuseppe Venturini"]
__license__ = "BSD 2-Clause License"
__version__ = '0.1-8'
__maintainer__ = "Giuseppe Venturini"
__email__ = "ggventurini+github@gmail.com"
__status__ = "Stable"

# Package testing can be done remotely, without display. This would make
# matplotlib fail (and consequently, the test itself).
# We check for $DISPLAY, but this makes us probably lose in portability,
# does Windows have this environment variable defined?
# Then again, who runs the test suit on a head-less windows machine...
# ... for the time being the following should be OK. If in the future that
# feature is needed by somebody, we can switch to
# if not os.system('python -c "import matplotlib.pyplot as plt;plt.figure()"')
import matplotlib
import os
if not ('DISPLAY' in os.environ or os.environ.get('READTHEDOCS', None)):
    matplotlib.use('Agg')

from ._DocumentNTF import DocumentNTF
from ._PlotExampleSpectrum import PlotExampleSpectrum
from ._SIunits import SIunits
from ._axisLabels import axisLabels
from ._bilogplot import bilogplot
from ._bplogsmooth import bplogsmooth
from ._bquantize import bquantize
from ._bunquantize import bunquantize
from ._calculateSNR import calculateSNR
from ._calculateTF import calculateTF
from ._cancelPZ import cancelPZ
from ._changeFig import changeFig
from ._circ_smooth import circ_smooth
from ._clans import clans
from ._constants import eps
from ._db import db
from ._dbm import dbm
from ._dbp import dbp
from ._dbv import dbv
from ._delay import delay
from ._ds_f1f2 import ds_f1f2
from ._ds_freq import ds_freq
from ._ds_hann import ds_hann
from ._ds_optzeros import ds_optzeros
from ._ds_quantize import ds_quantize
from ._ds_synNTFobj1 import ds_synNTFobj1
from ._dsclansNTF import dsclansNTF
from ._evalMixedTF import evalMixedTF
from ._evalRPoly import evalRPoly
from ._evalF0 import evalF0
from ._evalF1 import evalF1
from ._evalTF import evalTF
from ._evalTFP import evalTFP
from ._figureMagic import figureMagic
from ._frespF1 import frespF1
from ._impL1 import impL1
from ._infnorm import infnorm
from ._l1norm import l1norm
from ._logsmooth import logsmooth
from ._lollipop import lollipop
from ._mapABCD import mapABCD
from ._mapCtoD import mapCtoD
from ._mapQtoR import mapQtoR
from ._mapRtoQ import mapRtoQ
from ._mod1 import mod1
from ._mod2 import mod2
from ._nabsH import nabsH
from ._padb import padb
from ._padl import padl
from ._padr import padr
from ._padt import padt
from ._partitionABCD import partitionABCD
from ._peakSNR import peakSNR
from ._plotPZ import plotPZ
from ._plotSpectrum import plotSpectrum
from ._predictSNR import predictSNR
from ._pulse import pulse
from ._realizeNTF import realizeNTF
from ._realizeNTF_ct import realizeNTF_ct
from ._rms import rms
from ._rmsGain import rmsGain
from ._scaleABCD import scaleABCD
from ._simulateDSM import simulateDSM, simulation_backends
from ._simulateSNR import simulateSNR
from ._sinc_decimate import sinc_decimate
from ._stuffABCD import stuffABCD
from ._synthesizeChebyshevNTF import synthesizeChebyshevNTF
from ._synthesizeNTF import synthesizeNTF
from ._thermometer import thermometer
from ._undbm import undbm
from ._undbp import undbp
from ._undbv import undbv
from ._utils import circshift, cplxpair, mfloor, mround, pretty_lti, rat, gcd, lcm
from ._zinc import zinc



# -*- coding: utf-8 -*-
# test_utils.py
# This module provides the tests for the assorted functions in the utils
# module.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the assorted functions in the
_utils module.
"""

import unittest
import numpy as np
import deltasigma as ds

from deltasigma import synthesizeNTF

from deltasigma._utils import *
from deltasigma._utils import _get_zpk, _get_num_den, _cell_like_list
from deltasigma._constants import eps

def test_rat():
    """Test function for rat()"""
    import numpy.random as rnd
    for _ in range(10):
        n, d = rnd.randint(1, 5000), rnd.randint(1, 5000)
        fr = float(n) / float(d)
        nt, dt = rat(fr, tol=200e-6)
        assert np.allclose(
            (n / float(d) - nt / float(dt),), (0.,), atol=200e-6, rtol=1e-12)


def test_gcd_lcm():
    """Test function for gcd() and lcm"""
    a, b = 36, 31721
    tlcm, tgcd = 1141956, 1
    assert lcm(a, b) == tlcm
    assert gcd(a, b) == tgcd


def test_mfloor():
    """Test function for mfloor()"""
    tv = np.linspace(-1, 1, 10)
    tres = np.zeros(tv.shape)
    tres[tv >= 0] = np.floor(tv[tv >= 0])
    tres[tv < 0] = -np.ceil(np.abs(tv[tv < 0]))
    tresf = mfloor(tv)
    assert np.allclose(tres, tresf, atol=1e-8, rtol=1e-5)
    # w complex values.
    tv = [-1.9,  -0.2,  3.4,  5.6,  7.0,  2.4 + 3.6j]
    tres = [-2.0, -1.0, 3.0, 5.0, 7.0, (2 + 3j)]
    assert mfloor(tv) == tres


def test_carray():
    """Test function for carray()"""
    # don't touch 1d arrays
    a = np.arange(10)
    b = a.copy()
    assert np.all(carray(a) == b)
    # reshape 0-d arrays
    a = np.array(1)
    b = a.reshape((-1,))
    assert np.all(carray(a) == b)
    # convert lists and tuples to 1d arrays
    a = np.arange(10)
    c = a.tolist()
    b = tuple(c)
    assert np.all(a == carray(b))
    assert np.all(a == carray(c))
    # convert scalars to 1d arrays
    a = np.array((1,))
    b = 1
    assert np.all(carray(b) == a)


def test_cplxpair():
    """Test function for cplxpair()
    """
    a = np.array(
        [1 + eps * 20j, 1.1 + 2j, 1.1 - (2 + 50 * eps) * 1j, .1 + (1 + 99 * eps) * .2j, .1 - .2j])
    assert np.allclose(
        cplxpair(a), np.array(
            [0.1 - 0.2j, 0.1 + 0.2j, 1.1 - 2.j, 1.1 + 2.j, 1.0 + 0.j]),
        atol=100 * eps)
    # Mismatched imaginary part with correct sign
    a = (1, 1+2j, 1-2.4j) # this should fail. The complex sing. are not conj.
    try:
        cplxpair(a)
        # ValueError should be raised!
        assert False
    except ValueError:
        assert True
    # Mismatched real part with correct imaginary part
    a = (1, 1+2j, 2-2j) # this should fail. The complex sing. are not conj.
    try:
        cplxpair(a)
        # ValueError should be raised!
        assert False
    except ValueError:
        assert True
    # Mismatched real part with mismatched sign imaginary part
    a = (1, 1+2j, 2+2j) # this should fail. The complex sing. are not conj.
    try:
        cplxpair(a)
        # ValueError should be raised!
        assert False
    except ValueError:
        assert True

def test_diagonal_indices():
    """Test function for diagonal_indices()
    """
    a = np.arange(1, 26)
    a = a.reshape((5, 5))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 7, 13, 19, 25))
    assert np.allclose(dp1, (2, 8, 14, 20))
    assert np.allclose(dm1, (6, 12, 18, 24))
    a = np.arange(1, 21)
    a = a.reshape((4, 5))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 7, 13, 19))
    assert np.allclose(dp1, (2, 8, 14, 20))
    assert np.allclose(dm1, (6, 12, 18))
    a = a.reshape((5, 4))
    d = a[diagonal_indices(a)]
    dp1 = a[diagonal_indices(a, 1)]
    dm1 = a[diagonal_indices(a, -1)]
    assert np.allclose(d, (1, 6, 11, 16))
    assert np.allclose(dp1, (2, 7, 12))
    assert np.allclose(dm1, (5, 10, 15, 20))


def test_circshift():
    """Test function for circshift()
    """
    A = np.arange(1, 10).reshape((3, 3))
    shift = np.array((1, -1))
    Ashifted = circshift(A, shift)
    Ares = np.array(((8, 9, 7), (2, 3, 1), (5, 6, 4)))
    assert np.allclose(Ashifted, Ares)
    a = np.array([1, 2, 3, 4, 5]) # test for shift being a scalar
    assert np.allclose(circshift(a, 1), a[np.array((4, 0, 1, 2, 3), )])


def test_save_input_form():
    """Test function for save_input_form() and restore_input_form()"""
    a1 = np.arange(1, 26)
    a2 = a1.reshape((1, -1))
    form_a1 = save_input_form(a1)
    assert a1.shape == restore_input_form(a2, form_a1).shape
    assert np.allclose(a1, restore_input_form(a2, form_a1))
    b1 = 5
    b2 = carray(b1)
    form_b1 = save_input_form(b1)
    assert np.isscalar(restore_input_form(b2, form_b1))
    assert b1 == restore_input_form(b2, form_b1)


def test_get_zpk():
    """Test function for _get_zpk()"""
    from scipy.signal import zpk2ss
    z = (.4,)
    p = (.9, .1 + .2j, .1 - .2j)
    k = .4
    zt, pt, kt = _get_zpk(zpk2ss(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)
    zt, pt, kt = _get_zpk(zpk2tf(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)
    zt, pt, kt = _get_zpk(lti(z, p, k))
    assert np.allclose(zt, z, atol=1e-8, rtol=1e-5)
    assert np.allclose(pt, p, atol=1e-8, rtol=1e-5)
    assert np.allclose((kt, ), (k, ), atol=1e-8, rtol=1e-5)


def test_get_num_den():
    """Test function for _get_num_den()"""
    from scipy.signal import zpk2ss
    z = (.4,)
    p = (.9, .1 + .2j, .1 - .2j)
    k = .4
    num, den = zpk2tf(z, p, k)
    numt, dent = _get_num_den((num, den))  # num, den
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    numt, dent = _get_num_den((z, p, k))  # zpk
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    numt, dent = _get_num_den(lti(z, p, k))  # LTI
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    assert len(numt.shape) == 1
    assert len(dent.shape) == 1
    A, B, C, D = zpk2ss(z, p, k)  # A,B,C,D
    D = np.atleast_2d(D)
    numt, dent = _get_num_den((A, B, C, D))  # A,B,C,D
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    ABCD = np.vstack((
                     np.hstack((A, B)),
                     np.hstack((C, D))
                     ))
    numt, dent = _get_num_den(ABCD)  # A,B,C,D
    assert np.allclose(numt, num, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, den, atol=1e-8, rtol=1e-5)
    H = [[1],[2]] # check no 0-length arrays are returned
    numt, dent = _get_num_den(H)
    assert np.allclose(numt, 1, atol=1e-8, rtol=1e-5)
    assert np.allclose(dent, 2, atol=1e-8, rtol=1e-5)
    assert len(numt.shape) == 1
    assert len(dent.shape) == 1


def test_minreal():
    """Test function for minreal()"""
    from scipy.signal import zpk2ss
    z = np.array([0, 1])
    p = np.array([1, .5 + .5j, .5 - .5j])
    k = 1
    zt = z[:-1]
    pt = p[1:]
    pt.sort()
    inobj = (z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = zpk2tf(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = zpk2ss(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)
    inobj = lti(z, p, k)
    ltiobj = minreal(inobj)
    zeros, poles, gain = _get_zpk(ltiobj)
    poles.sort()
    assert np.allclose(zeros, zt, atol=1e-8, rtol=1e-5)
    assert np.allclose(poles, poles, atol=1e-8, rtol=1e-5)
    assert np.allclose((k,), (gain,), atol=1e-8, rtol=1e-5)


def test_pretty_lti():
    """Test function for pretty_lti()"""
    H = synthesizeNTF(5, 32, 1)
    tv = pretty_lti(H)
    res = '      (z^2 - 1.992z + 0.9999) (z^2 - 1.997z + 1) (z - 1)     \n' + \
          '-------------------------------------------------------------\n' + \
          ' (z^2 - 1.613z + 0.665) (z^2 - 1.796z + 0.8549) (z - 0.7778) '
    assert res == tv
    assert int(pretty_lti(([], [], 2))) == 2
    assert pretty_lti([[1], [], 2]) == '2 (z - 1)'
    assert pretty_lti([[], [.22222222], 2]) == \
        '      2       \n--------------\n (z - 0.2222) '
    assert pretty_lti(((0, 0, 1), (1, 2-1j, 2+1j, 2-1j, 2+1j), 5)) == \
        '         z^2 (z - 1)        \n' + \
        '5 --------------------------\n' + \
        '   (z^2 - 4z + 5)^2 (z - 1) '


def test_cell_like_list():
    """Test function for _cell_like_list()"""
    res = [[[1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1]],
           [[1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1]],
           [[1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1], [1, 1, 1, 1, 1]]]
    assert _cell_like_list((3, 4, 5), 1) == res


def test_mround():
    """Test function for mfloor()"""
    tv = np.linspace(-1, 1, 21)
    tres = np.zeros(tv.shape)
    tres[:6] = -1
    tres[-6:] = 1
    tresm = mround(tv)
    assert np.allclose(tres, tresm, atol=1e-8, rtol=1e-5)
    # w complex values.
    tv = [-1.9, -0.5, -0.2, 3.4, 4.5, 5.6, 7.0, 2.4 + 3.6j]
    tres = [-2.0, -1.0,  0.0, 3.0, 5.0, 6.0, 7.0, 2.0 + 4.0j]
    assert mround(tv) == tres



# -*- coding: utf-8 -*-
# test_undbv.py
# This module provides the tests for the undbv function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the undbv() function.
"""

import unittest
import numpy as np

from deltasigma import undbv

class TestUndbv(unittest.TestCase):
    """Test class for undbv()"""

    def setUp(self):
        pass

    def test_undbv_1(self):
        """Test function for undbv() 1/2"""
        self.assertTrue(np.allclose([undbv(53.05)], [449.26232467], rtol=1e-05,
                                    atol=1e-08))

    def test_undbv_2(self):
        """Test function for undbv() 2/2"""
        self.assertTrue(np.allclose([undbv(3)], [1.41253754462], rtol=1e-05,
                                    atol=1e-08))
        





# -*- coding: utf-8 -*-
# test_realizeNTF_ct.py
# This module provides the tests for the realizeNTF_ct function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the realizeNTF_ct() function.
"""

import unittest
import numpy as np
import deltasigma as ds

from nose.tools import raises

class TestRealizeNTF_CT(unittest.TestCase):
    """Test class for realizeNTF_ct()"""

    def setUp(self):
        # FB NRTZ 0 delay tdac default
        self.ntf = (np.array([1., 1.]), np.array([0., 0.]), 1)
        self.ABCDc_ref1 = np.array(((0, 0, 1, -1),
                                   (1, 0, 0, -1.5),
                                   (0, 1, 0, 0)), dtype=np.float)
        self.tdac2_ref1 = np.array(((-1, -1),
                                    (0,  1)))
        # FB NRTZ 0 delay tdac in short list form
        self.ABCDc_ref2 = np.array(((0, 0, 1, -1),
                                    (1, 0, 0, -1.5),
                                    (0, 1, 0, 0)), dtype=np.float)
        self.tdac2_ref2 = np.array(((-1, -1),
                                    (0,  1)))
        # FB multi-timing DAC
        self.ntf3 = (np.array([1., 1., 1]), np.array([0., 0., 0]), 1)
        self.tdac3 = [[1, 2], [1, 2], [[0.5, 1], [1, 1.5]], []]
        self.ABCDc_ref3 = np.array(((0., 0., 0., 1., -1., 0., 0., 0.),
                                    (1., 0., 0., 0., 0., -3., 0., 0.),
                                    (0., 1., 0., 0., 0., 0., -6., -2.6667),
                                    (0., 0., 1., 0., 0., 0., 0., 0.)))
        self.tdac2_ref3 = np.array(((-1., -1.),
                                    (1., 2.),
                                    (1., 2.),
                                    (0.5, 1.),
                                    (1., 1.5)))
        # test for FF
        self.ABCDc_ref4 = np.array([[0., 0., 1., -1.],
                                    [1., 0., 0.,  0.],
                                    [ 1.5, 1., 0., 0.]])
        self.tdac2_ref4 = np.array([[-1., -1.], [ 0., 1.]])
        # test for FF - NRZ DAC pulse delayed by .5
        self.ABCDc_ref5 = np.array([[0., 0., 1., -1.],
                                    [1., 0., 0., 0.],
                                    [2., 1., 0., -0.8750]])
        self.tdac2_ref5 = np.array([[-1., -1.], [ 0.5, 1.5]])

    def test_realizeNTF_ct_1(self):
        """Test function for realizeNTF_ct() 1/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FB')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref1, atol=1e-8,
                                    rtol=1e-5))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref1, atol=1e-8,
                                    rtol=1e-5))

    def test_realizeNTF_ct_2(self):
        """Test function for realizeNTF_ct() 2/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FB', tdac=[0, 1])
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref2, atol=1e-8,
                                    rtol=1e-5))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref2, atol=1e-8,
                                    rtol=1e-5))

    def test_realizeNTF_ct_3(self):
        """Test function for realizeNTF_ct() 3/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf3, 'FB', self.tdac3)
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref3, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref3, atol=1e-8,
                                    rtol=1e-4))

    def test_realizeNTF_ct_4(self):
        """Test function for realizeNTF_ct() 4/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref4, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref4, atol=1e-8,
                                    rtol=1e-4))

    def test_realizeNTF_ct_5(self):
        """Test function for realizeNTF_ct() 5/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FF', tdac=[.5, 1.5])
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref5, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref5, atol=1e-8,
                                    rtol=1e-4))

    @raises(ValueError)
    def test_realizeNTF_ct_6(self):
        """Test function for realizeNTF_ct() 6/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'DUMMY', tdac=[.5, 1.5])

    @raises(ValueError)
    def test_realizeNTF_ct_7(self):
        """Test function for realizeNTF_ct() 7/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FF', tdac=[.5, 1.5, 3.])

    @raises(ValueError)
    def test_realizeNTF_ct_8(self):
        """Test function for realizeNTF_ct() 8/15"""
        tdac= [[-1, -1], [.5, 1.5], [.5, .8], [0.1, 2.]]
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf3, 'FB', tdac)

    @raises(ValueError)
    def test_realizeNTF_ct_9(self):
        """Test function for realizeNTF_ct() 9/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf3, 'FF', self.tdac3)

    def test_realizeNTF_ct_10(self):
        """Test function for realizeNTF_ct() 10/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FB', method='NTF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref1, atol=1e-8,
                                    rtol=1e-5))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref1, atol=1e-8,
                                    rtol=1e-5))

    def test_realizeNTF_ct_11(self):
        """Test function for realizeNTF_ct() 11/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FB', tdac=[0, 1], method='NTF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref2, atol=1e-8,
                                    rtol=1e-5))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref2, atol=1e-8,
                                    rtol=1e-5))

    def test_realizeNTF_ct_12(self):
        """Test function for realizeNTF_ct() 12/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf3, 'FB', self.tdac3, method='NTF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref3, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref3, atol=1e-8,
                                    rtol=1e-4))

    def test_realizeNTF_ct_13(self):
        """Test function for realizeNTF_ct() 13/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FF', method='NTF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref4, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref4, atol=1e-8,
                                    rtol=1e-4))

    def test_realizeNTF_ct_14(self):
        """Test function for realizeNTF_ct() 14/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf, 'FF', tdac=[.5, 1.5],
                                        method='NTF')
        self.assertTrue(np.allclose(ABCDc, self.ABCDc_ref5, atol=1e-8,
                                    rtol=1e-4))
        self.assertTrue(np.allclose(tdac2, self.tdac2_ref5, atol=1e-8,
                                    rtol=1e-4))

    @raises(ValueError)
    def test_realizeNTF_ct_15(self):
        """Test function for realizeNTF_ct() 15/15"""
        ABCDc, tdac2 = ds.realizeNTF_ct(self.ntf3, 'FB', method='DUMMY')




# -*- coding: utf-8 -*-
# _infnorm.py
# This module provides the infnorm function.
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the infnorm() function, which finds the infinity
norm of a z-domain transfer function.
"""

from __future__ import division

from warnings import warn

import numpy as np
from scipy.optimize import fminbound

from ._evalTF import evalTF
from ._nabsH import nabsH


def infnorm(H):
    """Find the infinity norm of a z-domain transfer function.

    **Parameters:**

    H : object
        the LTI description of the DT system, which can be in one of the
        following forms:

        * an LTI object,
        * a zpk tuple,
        * a (num, den) tuple,
        * an ABCD matrix (internally converted to zpk representation),
        * a list-like containing the A, B, C, D matrices (also internally
          converted to zpk representation).

    **Returns:**

    Hinf : float
           The infinity norm of ``H``.
    fmax : float
           The frequency to which Hinf corresponds.
    """
    # Get a rough idea of the location of the maximum.
    N = 129
    w = np.linspace(0, 2*np.pi, num=N, endpoint=True)
    dw = 2*np.pi/(N-1)
    Hval = evalTF(H, np.exp(1j*w))
    Hinf = np.max(np.abs(Hval))
    wi = np.where(np.abs(Hval) == Hinf)[0]

    # Home in using the scipy "fminbound" function.
    # original MATLAB code:
    #   wmax = fminbnd(nabsH, w(wi)-dw, w(wi)+dw, options, H);
    wmax = fminbound(nabsH, w[wi]-dw, w[wi]+dw, args=(H,), \
                     xtol=1e-08, maxfun=5000, full_output=0)

    if wmax is None:
        warn('Hinf: scipy.optimize.fminbound() failed.'
             + ' The result returned may not be very accurate.')
        wmax = w[wi]

    Hinf = -nabsH(wmax, H);
    fmax = wmax/(2*np.pi);
    # in the original Toolbox, wmax is returned (seems to be never used though)
    # rep? We return fmax.
    return Hinf, fmax



# -*- coding: utf-8 -*-
# _partitionABCD.py
# Module providing the partitionABCD function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the partitionABCD() function
"""

def partitionABCD(ABCD, m=None):
    """Partition ABCD into A, B, C, D for an m-input state-space system.

    The :math:`ABCD` matrix is defined as:

    .. math::

        ABCD =
            \\left[
            \\begin{array}{c|c}
              A & B \\\\ \\hline
              C & D
            \\end{array}
            \\right].

    The matrices :math:`A`, :math:`B`, :math:`C` and :math:`D` define the
    evolution of a generic linear discrete time invariant system under study
    through the state-space representation:

    .. math::

        x(k + 1) = Ax(k) + Bu(k)

    .. math::

        y(k) = Cx(k) + Du(k)

    The matrices are:

    * :math:`A` is the state matrix, dimensions :math:`(n, n)`, :math:`n` being the number of states in the system,

    * :math:`B` is the input matrix, dimensions :math:`(n, m)`, :math:`m` being the number of inputs in the system,

    * :math:`C` is the output matrix, dimensions :math:`(r, n)`, :math:`r` being the number of outputs in the system,

    * :math:`D` is the feedthrough matrix, dimensions :math:`(r, m)`.

    The vectors are:

    * :math:`x(k)` the state sequence, of length :math:`n`,

    * :math:`u(k)`, the input sequence, of length :math:`m`, and

    * :math:`y(k)` is the output sequence, of length :math:`r`.

    **Parameters:**

    ABCD: ndarray
          The ABCD matrix to be partitioned

    m: int, optional
       The number of inputs in the system. It will be calculated from the ABCD matrix if not provided.

    **Returns:**

    A, B, C, D: tuple of ndarrays
                The partitioned matrices.

    .. seealso::

        :ref:`Modulator model: loop filter <loop-filter-label>` for a discussion of the ABCD matrix in the particular case of a delta-sigma modulator loop filter.

    """
    # remember the ABCD matrix is assembled like this:
    # [[A, B], 
    #  [C, D]]
    if m is None:
        n = min(ABCD.shape) - 1
        m = ABCD.shape[1] - n
    else:
        n = ABCD.shape[1] - m

    r = ABCD.shape[0] - n

    A = ABCD[:n, :n]
    B = ABCD[:n, n:n+m]
    C = ABCD[n:n+r, :n]
    D = ABCD[n:n+r, n:n+m]
    return A, B, C, D



# -*- coding: utf-8 -*-
# _simulateDSM.py
# Module providing the simulateDSM function,
# a switch to select the fastest simulation routine.
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.
#
# This file was originally from `pydsm`, then modified quite a bit.
# Many thanks to the original author.
#
# The original file is
# Copyright (c) 2012, Sergio Callegari
# All rights reserved.
#
# The modifications are mine.
# Copyright (c) 2014, G. Venturini and the python-deltasigma contributors
#

from __future__ import print_function

import os
from warnings import warn

import numpy as np

from ._config import _debug, setup_args
from ._simulateDSM_python import simulateDSM as _simulateDSM_python
from ._utils import _get_zpk, _is_zpk

warned = False

# Code to compile the Cython extensions
# Extensions tested on Linux and Mac OS X, but not on Windows
# please report any bug (or patches!) on
# https://github.com/ggventurini/python-deltasigma/issues

try:
    if 'nt' in os.name:
        # if somebody actually goes through the trouble of compiling
        # it on Windows, we'll make available a patch to re-enable it.
        # In most cases now, users only get error messages from BLAS
        # not being available.
        raise ImportError('CBLAS extension disabled on Windows')
    import pyximport
    pyximport.install(setup_args=setup_args)
    from ._simulateDSM_cblas import simulateDSM as _simulateDSM_cblas
except ImportError as e:
    if _debug:
        print(str(e))
    _simulateDSM_cblas = None

try:
    import pyximport
    pyximport.install(setup_args=setup_args, inplace=True)
    from ._simulateDSM_scipy_blas import simulateDSM as _simulateDSM_scipy_blas
except ImportError as e:
    if _debug:
        print(str(e))
    _simulateDSM_scipy_blas = None

# fall back to CPython

simulation_backends = {'CBLAS':(_simulateDSM_cblas is not None),
                       'Scipy_BLAS':(_simulateDSM_scipy_blas is not None),
                       'CPython':True}

def simulateDSM(u, arg2, nlev=2, x0=0.):
    """Simulate a Delta Sigma modulator

    Compute the output of a general delta-sigma modulator with input ``u``,
    a structure described by ``ABCD``, an initial state ``x0`` (default zero) and
    a quantizer with a number of levels specified by ``nlev``.

    **Syntax:**

     * ``[v, xn, xmax, y] = simulateDSM(u, ABCD, nlev=2, x0=0)``
     * ``[v, xn, xmax, y] = simulateDSM(u, ntf, nlev=2, x0=0)``

    **Parameters:**

    u : ndarray or sequence
        The input vector to be used in the simulation. Multiple inputs
        are implied by the number of rows in ``u``.
    arg2 : 2D ndarray or a supported LTI description
        The second argument may be either the ABCD matrix describing the
        modulator or its NTF. In the latter case, the NTF is converted to
        a ZPK description and the structure that is simulated is the
        block-diagonal structure used by scipy's ``zpk2ss()``.
        The STF is assumed to be 1.
    nlev : int or sequence or ndarray
        Number of levels in the quantizers. Set ``nlev`` to a scalar for a
        signle quantizer modulator. Multiple quantizers are implied by
        making nlev an array.
    x0 : float or sequence or ndarray
        The initial status of the modulator. If ``x0`` is set to float, its
        value will be used for all the states. If it is set to a sequence of
        floats, each of its values will be assigned to a state variable.

    **Returns:**

    v : ndarray
        The quantizer output.
    xn : ndarray
        The modulator states.
    xmax : nedarray
        The maximum value that each state reached during simulation.
    y : ndarray
        The quantizer input.

    **Notes:**

    Three implementations of this function are (potentially) available to the
    user, in order of ascending execution speed:

    * A CPython implementation, always available.
    * A Cython-based implementation requiring the BLAS headers and a compatible
      compiler.
    * A Cython-based implementation accessing the BLAS library pre-compiled
      through scipy, requiring only a compatible compiler.

    The difference in execution time from the first implementation -- dynamically
    interpreted -- to the latter two -- statically compiled automatically before
    execution -- is a factor 20.

    The fastest available implementation is automatically selected.

    To assess which implementations are available in your installation, check
    the ``simulation_backends`` variable, for example::

       from __future__ import print_function
       import deltasigma as ds
       print(ds.simulation_backends)

    Example output::

        {'Scipy_BLAS': True, 'CBLAS': True, 'CPython': True}


    **Example:**

    Simulate a 5th-order binary modulator with a half-scale sine-wave input and
    plot its output in the time and frequency domains.::

        import numpy as np
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        N = 8192
        fB = np.ceil(N/(2*OSR))
        f = 85
        u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
        v = simulateDSM(u, H)[0]

    Graphical display of the results:

    .. plot::

        import numpy as np
        import pylab as plt
        from numpy.fft import fft
        from deltasigma import *
        OSR = 32
        H = synthesizeNTF(5, OSR, 1)
        N = 8192
        fB = np.ceil(N/(2*OSR))
        f = 85
        u = 0.5*np.sin(2*np.pi*f/N*np.arange(N))
        v = simulateDSM(u, H)[0]
        plt.figure(figsize=(10, 7))
        plt.subplot(2, 1, 1)
        t = np.arange(85)
        # the equivalent of MATLAB 'stairs' is step in matplotlib
        plt.step(t, u[t], 'g', label='u(n)')
        plt.hold(True)
        plt.step(t, v[t], 'b', label='v(n)')
        plt.axis([0, 85, -1.2, 1.2]);
        plt.ylabel('u, v');
        plt.xlabel('sample')
        plt.legend()
        plt.subplot(2, 1, 2)
        spec = fft(v*ds_hann(N))/(N/4)
        plt.plot(np.linspace(0, 0.5, N/2 + 1), dbv(spec[:N/2 + 1]))
        plt.axis([0, 0.5, -120, 0])
        plt.grid(True)
        plt.ylabel('dBFS/NBW')
        snr = calculateSNR(spec[:fB], f)
        s = 'SNR = %4.1fdB' % snr
        plt.text(0.25, -90, s)
        s =  'NBW = %7.5f' % (1.5/N)
        plt.text(0.25, -110, s)
        plt.xlabel("frequency $1 \\\\rightarrow f_s$")

    Click on "Source" above to see the source code.
    """
    global warned
    if _simulateDSM_cblas or _simulateDSM_scipy_blas:
        if not _is_zpk(arg2) and not isinstance(arg2, np.ndarray):
            arg2 = _get_zpk(arg2)
        if _simulateDSM_cblas:
            return _simulateDSM_cblas(u, arg2, nlev, x0, store_xn=True,
                                      store_xmax=True, store_y=True)
        return _simulateDSM_scipy_blas(u, arg2, nlev, x0, store_xn=True,
                                       store_xmax=True, store_y=True)
    else:
        if not warned:
            warn('Using a slow implementation of simulateDSM\n' +
                 'Refer to the docs for how to switch to a fast one')
            warned = True
        return _simulateDSM_python(u, arg2, nlev, x0)



# -*- coding: utf-8 -*-
# _dsclansNTF.py
# Module providing the dsclansNTF function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the dsclansNTF() function
"""

import numpy as np

from ._utils import carray


def dsclansNTF(x, order, rmax, Hz):
    """ Conversion of clans parameters into a NTF.

    Translate x into H.
    I've changed the relationships between (zeta, wn) and x
    in order to guarantee LHP roots of the s-polynomial.
    
    Returns the NTF, a zpk tuple.
    """
    x = x.squeeze()
    Hz = carray(Hz)
    Hz = Hz.reshape((-1,))
    Hp = np.zeros((1,), dtype=np.complex128)
    odd = (order % 2 == 1)
    if odd:
        s = -x[0]**2.
        Hp[0] = rmax*(1. + s)/(1. - s)

    for i in range(0+1*odd, order, 2):
        Hp = np.hstack((Hp, np.zeros((2,))))
        zeta = x[i]**2
        wn = x[i + 1]**2
        s = np.roots(np.array((1, 2*zeta*wn, wn**2)))
        Hp[i:i+2] = rmax*(1. + s)/(1. - s)

    H = (Hz, Hp, 1.)
    return H



# -*- coding: utf-8 -*-
# _bilogplot.py
# Module providing the bilogplot function
# Copyright 2013 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""Module providing the bilogplot() function
"""

from __future__ import division

from warnings import warn

import numpy as np
import pylab as plt
from numpy.linalg import norm

from ._dbp import dbp
from ._utils import carray


def bilogplot(V, f0, fbin, x, y, **fmt):
    """Plot the spectrum of a band-pass modulator in dB.

    The plot is a logarithmic plot, centered in 0, corresponding to f0,
    extending to negative frequencies, with respect to the center frequencies
    and to positive frequencies.

    The plot employs a logarithmic x-axis transform far from the origin and a
    linear one close to it, allowing the x-axis to reach zero and extend to
    negative values as well.

    .. note::
        This is implemented in a slightly different way from The MATLAB Delta
        Sigma Toolbox, where all values below ``xmin`` are clipped and the scale is
        always logarithmic. It our implementation, no clippin is done and below
        ``xmin`` the data is simply plotted with a linear scale. For this reason
        slightly different plots may be generated.

    **Parameters:**

    V : 1d-ndarray or sequence
        Hann-windowed FFT

    f0 : int
        bin number of center frequency

    fbin : int
        bin number of test tone

    x : 3-elements sequence-like
        x is a sequence of three *positive* floats: ``xmin``, ``xmax_left``, ``xmax_right``.
        ``xmin`` is the minimum value of the logarithmic plot range. ``xmax_left`` is the
        length of the plotting interval on the left (negative) side, ``xmax_right`` is its
        respective on the right (positive) side.

    y : 3-elements sequence-like
        y is a sequence of three floats: ``ymin``, ``ymax``, ``dy``.
        ``ymin`` is the minimum value of the y-axis, ``ymax`` its maximum value and
        ``dy`` is the ticks spacing.

    .. note::
        The MATLAB Delta Sigma toolbox allows for a fourth option ``y_skip``, which
        is the ``incr`` value passed to MATLAB's ``axisLabels``.
        No such thing is supported here. A warning is issued if ``len(v) == 4``.

    Additional keyword parameters ``**fmt`` will be passed to matplotlib's ``semilogx()``.

    The FFT is smoothed before plotting and converted to dB. See
    :func:`logsmooth` for details regarding the algorithm used.

    **Returns:**

    *None*

    .. plot::

        from __future__ import division
        from deltasigma import synthesizeNTF, simulateDSM
        from deltasigma import calculateSNR, ds_hann, bilogplot
        import pylab as plt
        import numpy as np
        f0 = 1./8
        OSR = 64
        order = 8
        N = 8192
        H = synthesizeNTF(order, OSR, 1, 1.5, f0)
        fB = int(np.ceil(N/(2. * OSR)))
        ftest = int(np.round(f0*N + 1./3 * fB))
        u = 0.5*np.sin(2*np.pi*ftest/N*np.arange(N))
        v, xn, xmax, y = simulateDSM(u, H)
        spec = np.fft.fft(v*ds_hann(N))/(N/4)
        X = spec[:N/2 + 1]
        plt.figure()
        bilogplot(X, f0*N, ftest, (.03, .3, .3), (-140, 0, 10))

    """
    V = carray(V)
    if len(V.shape) > 1:
        if np.prod(V.shape) > max(V.shape):
            raise ValueError("The input value V should have only one" +
                             " non-unitary dimension.")
        V = V.squeeze()
    Xl = V[f0::-1]
    Xr = V[f0:]
    N = V.shape[0] - 1
    fbin = abs(fbin - f0)
    fl, pl = _logsmooth2(Xl, fbin)
    fr, pr = _logsmooth2(Xr, fbin)
    p = np.concatenate((pl[::-1], pr))
    f = np.concatenate((-fl[::-1], fr))
    plt.plot(f, p, **fmt)
    plt.xscale('symlog', linthreshx=x[0],
               subsx=np.logspace(10**int(np.ceil(np.log10(x[0]))),
                                 10**int(1+np.ceil(np.log10(max(x[2], x[1])))))
               )
    ax = plt.gca()
    ax.set_xlim([-x[1], x[2]])
    ax.set_ylim([y[0], y[1]])
    plt.grid(True)
    ytix = range(y[0], y[1] + 1, y[2])
    ax.yaxis.set_ticks(ytix)
    # we do not support axis labels
    # set_(gca,'YTickLabel', axisLabels(ytix, y[3]))
    #
    if len(y) == 4 and not y[3] is None:
        warn("Specifying y_skip is not currently supported and " +
             "it will be ignored. Sorry!")
    return


def _logsmooth2(X, inBin, nbin=8):
    """Smooth the fft, X, and convert it to dB.
    Use ``nbin`` bins from 0 to 3*inBin,
    thereafter increase bin sizes by a factor of 1.1, staying less than 2^10.
    For the 3 sets of bins inBin+[0:2], 2*inBin+[0:2], and
    3*inBin+[0:2], don't do averaging. This way, the noise BW
    and the scaling of the tone and its harmonics are unchanged.
    Unfortunately, harmonics above the third appear smaller than they
    really are because their energy is averaged over several bins.
    """
    N = max(X.shape)
    n = nbin
    f1 = int((inBin - 1) % n) + 1
    startbin = np.concatenate((np.arange(f1, inBin, n),
                               np.arange(inBin, inBin + 3),
                               np.arange(inBin + 3, 2*inBin, n),
                               2*inBin + np.arange(0, 3),
                               np.arange(2*inBin + 3, 3*inBin, n),
                               3*inBin + np.arange(0, 3)))
    m = startbin[-1] + n
    while m < N:
        startbin = np.concatenate((startbin, np.array((m,))))
        n = min(n*1.1, 2**10)
        m = np.round(m + n)
    stopbin = np.concatenate((startbin[1:] - 1, np.array((N,))))
    f = ((startbin + stopbin)/2. - 1)/N
    p = np.zeros(f.shape)
    for i in range(max(f.shape)):
        p[i] = 10*np.log10(norm(X[(startbin[i] - 1):stopbin[i]])**2 /
                           (stopbin[i] - startbin[i] + 1))
    return f, p




# -*- coding: utf-8 -*-
# test_undbp.py
# This module provides the tests for the undbp function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the undbp() function.
"""

import unittest
import numpy as np

from deltasigma import undbp

class TestUndbp(unittest.TestCase):
    """Test class for undbp()"""

    def setUp(self):
        pass

    def test_undbp_1(self):
        """Test function for undbp() 1/2"""
        self.assertTrue(np.allclose([undbp(53.05)], [201836.636368], rtol=1e-05,
                                    atol=1e-08))

    def test_undbp_2(self):
        """Test function for undbp() 2/2"""
        self.assertTrue(np.allclose([undbp(3)], [1.99526231497], rtol=1e-05,
                                    atol=1e-08))




# -*- coding: utf-8 -*-
# test_peakSNR.py
# This module provides the tests for the peakSNR function.
# Copyright 2014 Giuseppe Venturini
# This file is part of python-deltasigma.
#
# python-deltasigma is a 1:1 Python replacement of Richard Schreier's 
# MATLAB delta sigma toolbox (aka "delsigma"), upon which it is heavily based.
# The delta sigma toolbox is (c) 2009, Richard Schreier.
#
# python-deltasigma is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# LICENSE file for the licensing terms.

"""This module provides the test class for the peakSNR() function.
"""

import unittest
import pkg_resources

import numpy as np
import scipy.io

import deltasigma as ds

class TestPeakSNR(unittest.TestCase):
    """Test class for peakSNR()"""

    def setUp(self):
        ds._peakSNR._debug = True
        fname = pkg_resources.resource_filename(__name__, "test_data/test_peak_snr.mat")
        self.snr = scipy.io.loadmat(fname)['snr'].reshape((-1,))
        self.amp = scipy.io.loadmat(fname)['amp'].reshape((-1,))
        self.peak_snr, self.peak_amp = 76.612340603949761, -3.220409771005124

    def test_peakSNR(self):
        """Test function for peakSNR()"""
        ps, pa = ds.peakSNR(self.snr, self.amp)
        self.assertTrue(np.allclose(ps, self.peak_snr, atol=1e-8, rtol=1e-5))
        self.assertTrue(np.allclose(pa, self.peak_amp, atol=1e-8, rtol=1e-5))

